﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Globalization;

namespace Google.Charts {

    public abstract class ChartFill {

        protected ChartFill() {
            //...
        }

        public abstract override string ToString();

       
    }


    public class SolidFill : ChartFill {

        private Color _color;

        public SolidFill(Color color)
            : base()
        {
            this._color = color;
        }

        public Color Color { get { return _color; } }

        public override string ToString()
        {

            return "s," + Utils.ToHtml(_color);
        }
    }


    public class LinearGradientFill : ChartFill {

        protected internal struct ColorLocation
        {

            public ColorLocation(double coordinate, Color color)
                : this()
            {
                this.Color = color;
                this.Coordinate = coordinate;
            }

            public override string ToString()
            {
                return Utils.ToHtml(Color) + "," + this.Coordinate.ToString("0.0", CultureInfo.InvariantCulture);
            }


            public Color Color { get; private set; }
            public double Coordinate { get; private set; }
        }

        public LinearGradientFill()
            : base() {
            // ...
        }


        private int angle;

        public int Angle {
            get {
                return this.angle;
            }
            set {
                if (0 > value || value > 90) {
                    throw new ArgumentOutOfRangeException("value", "Angle must be between 0 and 90 degrees.");
                }
                this.angle = value;
            }
        }

        public void AddColor(double offset, Color color) {

            if (0 > offset || offset > 1) {
                throw new ArgumentOutOfRangeException("offset", "Offset must be between 0 and 1.");
            }

            colorLocation.Add(new ColorLocation(offset, color));
        }

        private List<ColorLocation> colorLocation = new List<ColorLocation>();

        protected void AppendColors(StringBuilder builder) {

            foreach (ColorLocation coLoc in colorLocation) {
                builder.Append("," + coLoc.ToString());
            }
        }

        public override string ToString() {
            StringBuilder result = new StringBuilder("lg," + Angle);

            AppendColors(result);

            return result.ToString();
        }
    }



    public class LinearStripes : LinearGradientFill {
     
        public LinearStripes()
            : base() {
            // ...
        }

        public override string ToString() {
            StringBuilder result = new StringBuilder("ls," + Angle);
            AppendColors(result);

            return result.ToString();
        }
    }
}
