﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web;

namespace Google.Charts {
   
    /// <summary>
    /// Represents a key-value pair used in the query string 
    /// send to the chart-provider.
    /// </summary>
    public class ChartParameter {


        /// <summary>
        /// Initializes a new instance of the <see cref="Chart&lt;T&gt;.ChartParameter"/> class.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="value">The value.</param>
        /// <exception cref="ArgumentNullException">Thrown when <paramref name="key"/> is 
        /// <see langword="null" /> or empty; or when <paramref name="value"/> is 
        /// <see langword="null" /> or empty.</exception>
        public ChartParameter(string key, string value) {

            Utils.ValidateNotNullOrEmpty(key, "key");
            Utils.ValidateNotNullOrEmpty(value, "value");

            this.Key = key;
            this.Value = value;
        }

        public string Key { get; private set; }
        public string Value { get; private set; }

        /// <summary>
        /// Renders the parameter as a key-valu pair.
        /// </summary>
        /// <returns>The parameter rendered as a key-value pair.
        /// </returns>
        public override string ToString() {
            return Key + "=" + Value ;
        }

        public virtual string ToQueryStringField()
        {
            return String.Format("{0}={1}", Key, HttpUtility.UrlEncode(Value));
        }

        public override bool Equals(object obj) {

            ChartParameter other = obj as ChartParameter;
            if (other == null) {
                return false;
            }

            // the parameters are equal to us if they have the same key.
            return string.Compare(other.Key, this.Key, StringComparison.OrdinalIgnoreCase) == 0;
        }

        public override int GetHashCode() {
            return Key.GetHashCode();
        }
    }
}
