﻿using System;
using System.Drawing;
using System.Text;

namespace Google.Charts
{
    /// <summary>
    /// Represents the title of the chart.
    /// </summary>
    public sealed class ChartTitle : ChartParameter
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ChartTitle"/> class.
        /// </summary>
        /// <param name="title">The title.</param>
        public ChartTitle(string title)
            : base("chtt", title)
        {
            //...
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ChartTitle"/> class.
        /// </summary>
        /// <param name="title">The title.</param>
        /// <param name="color">The color.</param>
        /// <param name="fontSize">Size of the font.</param>
        public ChartTitle(string title, Color? color, int? fontSize)
            : this(title)
        {
            this.Color = color;
            this.FontSize = fontSize;
        }

        public override string ToQueryStringField()
        {
            var title = base.ToQueryStringField();

            if (this.Color.HasValue)
            {
                return string.Concat(title, "&chts=", Utils.ToHtml(Color), ",", FontSize ?? 15); // 15 appears to be the default font size
            }

            return title;
        }

        /// <summary>
        /// Gets the title.
        /// </summary>
        /// <value>The title.</value>
        public string Title { get { return this.Value; } }

        /// <summary>
        /// Gets the color.
        /// </summary>
        /// <value>The color.</value>
        public Color? Color { get; private set; }

        /// <summary>
        /// Gets the size of the font.
        /// </summary>
        /// <value>The size of the font.</value>
        public int? FontSize { get; private set; }


        /// <summary>
        /// Performs an implicit conversion from <see cref="System.String"/> 
        /// to <see cref="Google.Charts.ChartTitle"/>.
        /// </summary>
        /// <param name="value">The string to convert.</param>
        /// <returns>The result of the conversion.</returns>
        public static implicit operator ChartTitle(string value)
        {
            if (String.IsNullOrEmpty(value))
            {
                return null;
            }

            return new ChartTitle(value);
        }

        /// <summary>
        /// Creates a new <see cref="Google.Charts.ChartTitle"/> from the specified string.
        /// </summary>
        /// <param name="value">The string to convert.</param>
        /// <returns>A new <see cref="Google.Charts.ChartTitle"/> or <see langword="null" /> if
        /// <paramref name="value"/> is <see langword="null" /> or an empty string.</returns>
        public static ChartTitle FromString(string value)
        {
            return (ChartTitle)value;
        }
    }
}
