﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Google.Charts 
{
    public abstract class BarChart : ColoredChart<DataPoint>
    {
        protected BarChart() { }

        private Direction direction;
        /// <summary>
        /// Gets or sets the direction of the chart.
        /// </summary>
        /// <value>The direction of the chart.</value>
        public Direction Direction
        {
            get { return this.direction; }
            set { this.direction = value; }
        }

        private int? barSize;
        /// <summary>
        /// Gets or sets the size of the bar.
        /// </summary>
        /// <value>The size of the bar.</value>
        public int BarSize { get { return (int)this.barSize; } set { this.barSize = value; } }

        private int? spacing;
        /// <summary>
        /// Gets or sets the spacing between bars.
        /// </summary>
        /// <value>The spacing between bars.</value>
        public int Spacing { get { return (int)this.spacing; } set { this.spacing= value; } }

        /// <summary>
        /// Gets the data parameter.
        /// </summary>
        /// <returns></returns>
        /// <exception cref="InvalidOperationException">Thrown when no data is available in the 
        /// <see cref="P:Data"/> property.</exception>
        protected virtual void CreateDataParameter() {

            if (Data == null || Data.Count < 1) {
                throw new InvalidOperationException("No data specified");
            }

            DataEncoder encoder = DataEncoder.GetEncoder(this.Resolution);

            string[] result = new string[Data.Count];

            for (int i = 0; i < Data.Count; i++) {
                result[i] = Data[i].ToDataString(encoder);
            }

            base.AddParameter( new ChartParameter("chd",  string.Format("{0}:{1}", encoder.Identifier, string.Join(encoder.SetSeparator, result))));
        }

        protected virtual void CreateBarThicknessParameter() {

            if (barSize.HasValue || spacing.HasValue) {

                // 25 is the default barsize
                string result = barSize.HasValue ? barSize.ToString() : "25";

                // spacing is optional
                if (spacing.HasValue) {
                    result += "," + spacing.ToString();
                }

                base.AddParameter(new ChartParameter("chbh", result));
            }
        }

        protected override void CreateParameters() {
            base.CreateParameters();
            CreateDataParameter();
            CreateBarThicknessParameter();
        }
    }

    /// <summary>
    /// Defines the direction of the chart.
    /// </summary>
    public enum Direction {
        /// <summary>
        /// Indicated that the chart is rendered horizontal.
        /// </summary>
        Horizontal,

        /// <summary>
        /// Indicates that the chart is rendered vertical.
        /// </summary>
        Vertical,
    }
}
