﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Google.Charts.Lines
{
    public abstract class LineChartBase<T> : Chart<T>
        where T : DataPoint/*, new()*/
    {

        private List<LabelAxis> _labels = new List<LabelAxis>();
        private Color _color;

        public Color FillColor
        {
            get { return _color; }
            set { _color = value; }
        }

        public List<LabelAxis> Labels
        {
            get { return _labels; }
            protected set { _labels = value; }
        }

        protected string type = "lxy";

        public List<LabelDesc> AddLabel(LabelType type)
        {
            LabelAxis la = new LabelAxis(type);
            _labels.Add(la);
            return la.Labels;
        }

        protected virtual void CreateDataLabelsParameter()
        {
            if (_labels.Count > 0)
            {
                StringBuilder chxt = new StringBuilder();
                StringBuilder chxl = new StringBuilder();
                StringBuilder chxp = new StringBuilder();
                for (int i = 0; i < _labels.Count; ++i)
                {
                    LabelAxis la = _labels[i];
                    switch (la.Type)
                    {
                        case LabelType.BottomX:
                            chxt.Append("x");
                            break;
                        case LabelType.LeftY:
                            chxt.Append("y");
                            break;
                        case LabelType.RightY:
                            chxt.Append("r");
                            break;
                        case LabelType.TopX:
                            chxt.Append("t");
                            break;
                    }
                    chxt.Append(",");
                    if (la.Labels.Count > 0)
                    {
                        chxl.Append(i).Append(":|");
                        int l = chxp.Length;
                        chxp.Append(i).Append(",");
                        bool has_pos = false;
                        foreach (LabelDesc ld in la.Labels)
                        {
                            chxl.Append(ld.Label).Append("|");
                            if (ld.Position.HasValue)
                            {
                                has_pos = true;
                                chxp.Append(ld.Position.Value.ToString("F1",System.Globalization.CultureInfo.InvariantCulture)).Append(",");
                            }
                        }
                        if (has_pos)
                        {
                            chxp.Length--;
                            chxp.Append("|");
                        }
                        else
                        {
                            chxp.Length = l;
                        }
                    }
                }
                chxt.Length--;
                base.AddParameter(new ChartParameter("chxt", chxt.ToString()));
                if (chxl.Length > 0)
                {
                    chxl.Length--;
                    base.AddParameter(new ChartParameter("chxl", chxl.ToString()));
                }
                if (chxp.Length > 0)
                {
                    chxp.Length--;
                    base.AddParameter(new ChartParameter("chxp", chxp.ToString()));
                }
            }
        }

        /// <summary>
        /// Gets the querystring parameter which describes the type class.
        /// </summary>
        /// <returns>The type parameter.</returns>
        protected virtual void CreateTypeParameter()
        {
            ChartParameter typeParam = new ChartParameter("cht", this.type);
            base.AddParameter(typeParam);
        }

        protected virtual void CreateColorParameter()
        {
            StringBuilder chco = new StringBuilder();
            for (int i = 0; i < Data.Count; i++)
            {
                Color c = Data[i].Color;
                if (!c.IsEmpty)
                    chco.Append(Utils.ToHtml(c)).Append(",");
            }
            if (chco.Length > 0)
            {
                chco.Length--;
                AddParameter(new ChartParameter("chco", chco.ToString()));
            }
        }

        protected abstract void CreateDataParameter();

        protected override void CreateParameters()
        {
            CreateTypeParameter();
            CreateDataParameter();
            CreateDataLabelsParameter();
            if (!_color.IsEmpty)
            {
                ChartParameter typeParam = new ChartParameter("chm", "B," + Utils.ToHtml(_color) + ",0,0,0");
                base.AddParameter(typeParam);
            }
            CreateColorParameter();
        }
    }
}
