﻿using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Drawing;
using System.Collections.Generic;

namespace Google.Charts {

    public class SimpleLineChart : Google.Charts.Lines.LineChartBase<LabeledDataPoint>
    {
        bool autoLabel;

        /// <summary>
        /// Initializes a new instance of the <see cref="SimpleLineChart"/> class.
        /// </summary>
        public SimpleLineChart()
            : base() 
        {
            this.type = "lc";
            autoLabel = true;
        }

        public DataSet<LabeledDataPoint> AddLine(params double[] values)
        {
            return AddLine(null, values);
        }

        public DataSet<LabeledDataPoint> AddLine(string label, params double[] values)
        {
            return AddLine(label, new Color(), values);
        }

        public DataSet<LabeledDataPoint> AddLine(Color color, params double[] values)
        {
            return AddLine(null, color, values);
        }

        public DataSet<LabeledDataPoint> AddLine(string label, Color color, params double[] values)
        {
            DataSet<LabeledDataPoint> ds = AddLine(label, color);

            foreach (double v in values)
            {
                ds.Add(new LabeledDataPoint(v, null));
            }

            return ds;
        }

        public DataSet<LabeledDataPoint> AddLine(params float[] values)
        {
            return AddLine(null, values);
        }

        public DataSet<LabeledDataPoint> AddLine(string label, params float[] values)
        {
            return AddLine(label, new Color(), values);
        }

        public DataSet<LabeledDataPoint> AddLine(Color color, params float[] values)
        {
            return AddLine(null, color, values);
        }

        public DataSet<LabeledDataPoint> AddLine(string label, Color color, params float[] values)
        {
            DataSet<LabeledDataPoint> ds = AddLine(label, color);

            foreach (float v in values)
            {
                ds.Add(new LabeledDataPoint(v, null));
            }

            return ds;
        }

        public DataSet<LabeledDataPoint> AddLine(string label, Color color, params LabeledDataPoint[] values)
        {
            DataSet<LabeledDataPoint> ds = AddLine(label, color);

            foreach (LabeledDataPoint v in values)
            {
                ds.Add(v);
            }

            return ds;
        }

        public DataSet<LabeledDataPoint> AddLine(params LabeledDataPoint[] values)
        {
            return AddLine(null, new Color(), values);
        }

        public DataSet<LabeledDataPoint> AddLine(string label, Color color)
        {
            DataSet<LabeledDataPoint> ds = new DataSet<LabeledDataPoint>();
            Data.Add(ds);

            if (!color.IsEmpty)
                ds.Color = color;

            ds.Label = label;

            return ds;
        }

        public bool AutoLabel
        {
            get { return autoLabel; }
            set { autoLabel = value; }
        }

        protected override void CreateDataParameter()
        {

            if (Data == null || Data.Count < 1) {
                throw new InvalidOperationException("No data specified");
            }

            //DataEncoder encoder = DataEncoder.SimpleEncoder;
            DataEncoder encoder = DataEncoder.GetEncoder(this.Resolution);

            string[] result = new string[Data.Count];
            if (Data.Count > 1)
            {
                double max = double.MinValue;
                double min = double.MaxValue;
                for (int i = 0; i < Data.Count; i++)
                {
                    if (max < Data[i].MaxValue)
                        max = Data[i].MaxValue;
                    if (min > Data[i].MinValue)
                        min = Data[i].MinValue;
                }
                for (int i = 0; i < Data.Count; i++)
                {
                    result[i] = Data[i].ToDataString(encoder, max, min);
                }
            }
            else
                for (int i = 0; i < Data.Count; i++)
                {
                    result[i] = Data[i].ToDataString(encoder);
                }

            base.AddParameter( new ChartParameter("chd", String.Format("{0}:{1}", encoder.Identifier, string.Join(encoder.SetSeparator, result))));
        }

        protected override void CreateDataLabelsParameter()
        {

            if (Labels.Count == 0 && autoLabel)
            {
                List<LabelAxis> l = new List<LabelAxis>();
                LabelAxis la = new LabelAxis(LabelType.BottomX);
                for (int i = 0; i < Data[0].DataPoints.Length; i++)
                {
                    LabelDesc ld = new LabelDesc(Data[0].DataPoints[i].Label);
                    la.Labels.Add(ld);
                }
                l.Add(la);
                Labels = l;
            }
            base.CreateDataLabelsParameter();
        }
    }
}
