﻿using System;
using System.Drawing;

namespace Google.Charts {

    /// <summary>
    /// Represents a 2-dimensional piechart.
    /// </summary>
    public class PieChart : ColoredChart<LabeledDataPoint> {

        /// <summary>
        /// Initializes a new instance of the <see cref="PieChart"/> class.
        /// </summary>
        public PieChart() 
            :base() 
        {
//
        }

        /// <summary>
        /// Adds a segment to the chart.
        /// </summary>
        /// <param name="value">The value.</param>
        public void AddSegment(double value) {

            AddSegment(value, null, null);
        }

        /// <summary>
        /// Adds a segment to the chart.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <param name="label">The label.</param>
        public void AddSegment(double value, string label) {

            AddSegment(value, label, null);
        }


        /// <summary>
        /// Adds a segment to the chart.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <param name="label">The label.</param>
        /// <param name="color">The color.</param>
        public void AddSegment(double value, string label, Color? color) 
        {
            if (value <= 0)
            {
                throw new ArgumentOutOfRangeException("value", "value must be greater than 0.");
            }

            DataSet<LabeledDataPoint> ds = GetFirstDS();
            LabeledDataPoint point = new LabeledDataPoint(value, label);
            if (color.HasValue)
            {
                Colors.Add(color.Value);
                int idx = Colors.Count - 1;
                ds.Insert(idx, point);
            }
            else
                ds.Add(point);
        }


        /// <summary>
        /// Gets the querystring parameter which describes the type class.
        /// </summary>
        /// <returns>The type parameter.</returns>
        protected virtual void CreateTypeParameter() {

            ChartParameter typeParam =  new ChartParameter("cht", "p");
            base.AddParameter(typeParam);
        }

        protected virtual void CreateDataParameter() {

            if (Data == null || Data.Count ==0) {
                throw new InvalidOperationException("No data specified");
            }

            DataEncoder encoder = DataEncoder.GetEncoder(this.Resolution);

            ChartParameter dataParam = new ChartParameter( "chd", String.Format("{0}:{1}", encoder.Identifier, Data[0].ToDataString(encoder)));

            base.AddParameter(dataParam);
        }

        protected virtual void CreateDataLabelsParameter() {

            if (Data == null || Data.Count < 1) {
                throw new InvalidOperationException("No data specified");
            }

            string[] result = new string[Data[0].DataPoints.Length];
            for (int i = 0; i < Data[0].DataPoints.Length; i++) {

                result[i] = Data[0].DataPoints[i].Label;
            }

            ChartParameter labelParam = new ChartParameter("chl", String.Join("|", result));
            base.AddParameter(labelParam);
        }

        protected override void CreateParameters() {
            base.CreateParameters();
            CreateTypeParameter();
            CreateDataParameter();
            CreateDataLabelsParameter();
            //CreateColorsParameter();
        }
    }
}
