﻿using System;
using System.Collections.Generic;

using System.Text;

namespace Google.Charts {

    public class ScatterPlot : Chart<TripleDataPoint> {

        /// <summary>
        /// Initializes a new instance of the <see cref="ScatterPlot"/> class.
        /// </summary>
        public ScatterPlot()
            : base() {
            // the scatter plot only supports one set of data
            // each datapoint consists of an x and a y value and an optional size.
            //this.Data = new DataSet<TripleDataPoint>[1] { new DataSet<TripleDataPoint>() };
        }

        public void AddPoint(double x, double y) {

            // indicate missing value for size
            AddPoint(x, y, -1);
        }

        public void AddPoint(double x, double y, double size) {

            // datavalidation is handled in the encoding
            GetFirstDS().Add(new TripleDataPoint(x, y, size));
        }


        /// <summary>
        /// Gets the querystring parameter which describes the type class.
        /// </summary>
        /// <returns>The type parameter.</returns>
        protected virtual void CreateTypeParameter() {

            ChartParameter typeParam = new ChartParameter("cht", "s");
            base.AddParameter(typeParam);
        }

        protected virtual void CreateDataParameter() {

            if (Data == null || Data.Count < 1) {
                throw new InvalidOperationException("No data specified");
            }

            DataEncoder encoder = DataEncoder.GetEncoder(this.Resolution);

            string[] result = new string[3];

            StringBuilder x = new StringBuilder();
            StringBuilder y = new StringBuilder();
            StringBuilder w = new StringBuilder();

            double max = Data[0].MaxValue;

            foreach (TripleDataPoint point in Data[0]) {

                x.Append(encoder.EncodeValue(point.XValue, max));
                y.Append(encoder.EncodeValue(point.Value, max));
                w.Append(encoder.EncodeValue(point.Weight, max));
            }

            result[0] = x.ToString();
            result[1] = y.ToString();
            result[2] = w.ToString();


           base.AddParameter( new ChartParameter("chd",  string.Format("{0}:{1}", encoder.Identifier, string.Join(encoder.SetSeparator, result))));
        }

        protected override void CreateParameters()
        {
            CreateTypeParameter();
            CreateDataParameter();
        }
    }
}

