﻿using System;
using System.Drawing;
using System.Diagnostics.CodeAnalysis;

namespace Google.Charts {
    
    /// <summary>
    /// Represents a circle in a venn diagram.
    /// </summary>
    public sealed class VennCircle : DataPoint {

        /// <summary>
        /// Initializes a new instance of the <see cref="VennCircle"/> class.
        /// </summary>
        /// <param name="size">The size.</param>
        public VennCircle(double size)
            : this(size, null, null) {
            // ...
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="VennCircle"/> class.
        /// </summary>
        /// <param name="size">The size.</param>
        /// <param name="label">The label.</param>
        public VennCircle(double size, string label)
            : this(size, label, null) {
            //...
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="VennCircle"/> class.
        /// </summary>
        /// <param name="size">The size.</param>
        /// <param name="label">The label.</param>
        /// <param name="color">The color.</param>
        public VennCircle(double size, string label, Color? color) {

            if (size < 0) {
                throw new ArgumentOutOfRangeException("size", "size must be greater than zero.");
            }

            this.Value = size;
            this.Label = label;
            this.Color = color;
        }


        /// <summary>
        /// Represents a <see cref="VennCircle"/> with size 0.
        /// </summary>
        /// <remarks>This circle is not shown in the chart.</remarks>
        [SuppressMessage("Microsoft.Security", "CA2104:DoNotDeclareReadOnlyMutableReferenceTypes",Justification="VennCircle is immutable.")]
        public static readonly VennCircle Empty = new VennCircle(0);


        /// <summary>
        /// Gets or sets the size.
        /// </summary>
        /// <value>The size.</value>
//        public double Size { get; private set; }
        /// <summary>
        /// Gets or sets the label.
        /// </summary>
        /// <value>The label.</value>
        public string Label { get; private set; }
        /// <summary>
        /// Gets or sets the color.
        /// </summary>
        /// <value>The color.</value>
        public Color? Color { get; private set; }


        /// <summary>
        /// Creates a new <see cref="Google.Charts.VennCircle"/> using the 
        /// specifed <paramref name="size"/> as it's size.
        /// </summary>
        /// <param name="size">The size.</param>
        /// <returns>The created VennCircle.</returns>
        public static VennCircle FromDouble(double size) {

            return (VennCircle)size;
        }

        /// <summary>
        /// Performs an implicit conversion from <see cref="System.Double"/> 
        /// to <see cref="Google.Charts.VennCircle"/>.
        /// </summary>
        /// <param name="size">The size.</param>
        /// <returns>The result of the conversion.</returns>
        public static implicit operator VennCircle(double size) {

            return new VennCircle(size);
        }
    }
}
