﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Google.Charts {

    /// <summary>
    /// Represents a Venn diagram consisting of three circles.
    /// </summary>
    /// todo: overlap should max 100%
    /// todo: use collection for the circles
    /// todo: max should only be used for the size of the circles
    
    public class VennDiagram : ColoredChart<VennCircle> {

        /// <summary>
        /// Initializes a new instance of the <see cref="VennDiagram"/> class.
        /// </summary>
        public VennDiagram()
            : base() {
            //....
        }

        /// <summary>
        /// Gets the querystring parameter which describes the type class.
        /// </summary>
        /// <returns>The type parameter.</returns>
        protected virtual void CreateTypeParameter() {

            ChartParameter typeParam = new ChartParameter("cht", "v");
            base.AddParameter(typeParam);
        }

        protected virtual void CreateDataParameter() {

            DataEncoder encoder = DataEncoder.GetEncoder(this.Resolution);

            string[] result = new string[7];
            result[0] = encoder.EncodeValue(circleA.Value, max);
            result[1] = encoder.EncodeValue(circleB.Value, max);
            result[2] = encoder.EncodeValue(circleC.Value, max);

            result[3] = encoder.EncodeValue(intersectionAB, 100);
            result[4] = encoder.EncodeValue(intersectionBC, 100);
            result[5] = encoder.EncodeValue(intersectionCA, 100);

            result[6] = encoder.EncodeValue(intersectionABC, 100);

            base.AddParameter( new ChartParameter( "chd", string.Format("{0}:{1}", encoder.Identifier, string.Join(encoder.Separator, result))));
        }

        protected virtual void CreateDataLabelsParameter() {

            StringBuilder result = new StringBuilder();
            if (!String.IsNullOrEmpty(circleA.Label)) {
                result.Append(circleA.Label);
            }

            if (!String.IsNullOrEmpty(circleB.Label)) {
                if (result.Length > 0) {
                    result.Append("|");
                }

                result.Append(circleB.Label);
            }

            if (!String.IsNullOrEmpty(circleC.Label)) {
                if (result.Length > 0) {
                    result.Append("|");
                }
                result.Append(circleC.Label);
            }

            if (result.Length == 0) {
                return;
            }


            ChartParameter labelsParameter = new ChartParameter("chdl", result.ToString());
            
            base.AddParameter( labelsParameter);
        }


        protected virtual void CreateDataSetColorsParameter() {
            
            //StringBuilder c = new StringBuilder();
            //if( circleA.Color != null ){
            //    c.Append(Utils.ToHtml(circleA.Color));
            //}
            
            //if (circleB.Color != null) {
            //    if (c.Length > 0) {
            //        c.Append(",");
            //    }

            //    c.Append(Utils.ToHtml(circleB.Color));
            //}

            //if (circleC.Color != null) {
            //    if (c.Length > 0) {
            //        c.Append(",");
            //    }
            //    c.Append(Utils.ToHtml(circleC.Color));
            //}

            //if (c.Length == 0) {
            //    return;
            //}

            //ChartParameter colors = new ChartParameter("chco", c.ToString() );

            //base.AddParameter(colors);
            if (circleA.Color != null)
                Colors.Add(circleA.Color.Value);
            if (circleB.Color != null)
                Colors.Add(circleB.Color.Value);
            if (circleC.Color != null)
                Colors.Add(circleC.Color.Value);
        }

        protected override void CreateParameters() {
            CreateDataSetColorsParameter();
            base.CreateParameters();
            CreateDataLabelsParameter();
            CreateTypeParameter();
            CreateDataParameter();
        }

        private double max;

        /// <summary>
        /// Gets or sets the size of circle A.
        /// </summary>
        /// <value>The size of circle A.</value>
        public VennCircle CircleA {
            get { return circleA; }
            set {
                if (value == null) {
                    throw new ArgumentNullException("value");
                }

                if (value.Value > max) { max = value.Value; }
                circleA = value;
            }
        }
        private VennCircle circleA;

        /// <summary>
        /// Gets or sets the size of circle B.
        /// </summary>
        /// <value>The size of circle B.</value>
        public VennCircle CircleB {
            get { return circleB; }
            set {
                if (value == null) {
                    throw new ArgumentNullException("value");
                } 
                
                if (value.Value > max) { max = value.Value; }
                circleB = value;
            }
        }
        private VennCircle circleB;

        /// <summary>
        /// Gets or sets the size of circle C.
        /// </summary>
        /// <value>The size of circle C.</value>
        public VennCircle CircleC {
            get { return circleC; }
            set {
                if (value == null) {
                    throw new ArgumentNullException("value");
                } 
                
                if (value.Value > max) { max = value.Value; }
                circleC = value;
            }
        }
        private VennCircle circleC;

        /// <summary>
        /// Gets or sets the area of intersection between circles A and B.
        /// </summary>
        /// <value>The intersection between A and B.</value>
        public int IntersectionAB {
            get { return intersectionAB; }
            set {
                if (value > max) { max = value; }
                intersectionAB = value;
            }
        }
        private int intersectionAB;

        /// <summary>
        /// Gets or sets the area of intersection between circles B and C.
        /// </summary>
        /// <value>The intersection between B and C.</value>
        public int IntersectionBC {
            get { return intersectionBC; }
            set {
                if (value > max) { max = value; }
                intersectionBC = value;
            }
        }
        private int intersectionBC;

        /// <summary>
        /// Gets or sets the area of intersection between circles C and A.
        /// </summary>
        /// <value>The intersection between C and A.</value>
        public int IntersectionCA {
            get { return intersectionCA; }
            set {
                if (value > max) { max = value; }
                intersectionCA = value;
            }
        }
        private int intersectionCA;

        /// <summary>
        /// Gets or sets the area of intersection between all three circles A, B and C.
        /// </summary>
        /// <value>The intersection between A, B and C.</value>
        public int IntersectionABC {
            get { return intersectionABC; }
            set {
                if (value > max) { max = value; }
                intersectionABC = value;
            }
        }
        private int intersectionABC;
    }
}

