﻿using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Collections.Generic;
using System.Drawing;
using System.Globalization;
using System.Text;

namespace Google.Charts {

    /// <summary>
    /// Represents the data for a chart. Containes the date for a single line in a linechart,
    /// or all data for a PieChart.
    /// </summary>
    public class DataSet<T> : IEnumerable<T> where T : DataPoint/*, new() */{

        private string _label;

        public string Label
        {
            get { return _label; }
            set { _label = value; }
        }
        private Color _color;

        public Color Color
        {
            get { return _color; }
            set { _color = value; }
        }

        private double _maxValue;

        public double MaxValue
        {
            get { return _maxValue; }
            set { _maxValue = value;}
        }

        private double _minValue = double.MaxValue;

        public double MinValue
        {
            get { return _minValue; }
            set { _minValue = value; }
        }

        public DataSet() 
        {
            // ...
        }

        //public string Label { get; set; }

        //public Color Color { get; set; }

        private List<T> dataPoints = new List<T>();

        public T[] DataPoints {
            get {
                return this.dataPoints.ToArray();
            }
        }

        //public double MaxValue { get; private set; }

/*
        public T MaxDataPoint {
            get;
            private set;
        }

        public T MinDataPoint {
            get;
            private set;
        }
  */

        public IEnumerator<T> GetEnumerator() {

            if (DataPoints == null) {
                throw new InvalidOperationException("No datapoints defined.");
            }

            return dataPoints.GetEnumerator();
        }

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator() {
            return this.GetEnumerator();
        }

        public void Insert(int idx, T data)
        {
            SetMinMax(data);
            dataPoints.Insert(idx,data);
        }

        public void Add(T data) {

            SetMinMax(data);
            this.dataPoints.Add(data);
        }

        private void SetMinMax(T data)
        {
            if (data.Max > _maxValue)
            {
                _maxValue = data.Max;
            }

            if (data.Min < _minValue)
            {
                this._minValue = data.Min;
            }
        }


        public string ToDataString(DataEncoder encoder) 
        {
            return ToDataString(encoder, MaxValue, MinValue);
        }

        public string ToDataString(DataEncoder encoder, double max, double min)
        {

            //string[] result = new string[dataPoints.Count];
            //for (int i = 0; i < dataPoints.Count; i++) {
            //    result[i] = encoder.EncodeValue(dataPoints[i].Value, this.MaxValue);
            //}

            //return string.Join(encoder.Separator, result);
            bool ap = !string.IsNullOrEmpty(encoder.Separator);
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < dataPoints.Count; i++)
            {
                sb.Append(encoder.EncodeValue(dataPoints[i].Value, max, min));
                if (ap)
                    sb.Append(encoder.Separator);
            }
            if (ap && sb.Length > 0) sb.Length--;
            return sb.ToString();
        }

        public override string ToString() {

            return "DataSet {Label:" + Label + "}";
        }
    }
}
