﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Google.Charts {

    public sealed class ExtendedEncoder : DataEncoder {

        internal ExtendedEncoder()
            : base() {

            this.MissingValue = "__";
            this.SetSeparator = ",";
        }

        public override string EncodeValue(double value, double maxValue, double minValue)
        {

            if (value > maxValue) {
                throw new ArgumentOutOfRangeException("value", "value must be less than maxValue");
            }

            if (value < minValue)
            {
                throw new ArgumentOutOfRangeException("value", "value must be greater than minValue");
            }

            if( !IsValid(value)){
                return MissingValue;
            }

            if (maxValue == minValue)
                minValue = 0;

            double k = rangeMax / (maxValue - minValue);
            // maps the value to a new value inside the range (0,4095)
            int inRange = (int)Math.Round((value-minValue) * k);
            if (inRange > 0) inRange--;

            // converts the mapped value to it's 64 bit representation.
            char[] result = new char[2];
            result[0] = values[inRange / 64];
            result[1] = values[inRange % 64];

            return new string(result);
        }

        public override string Identifier {
            get { return "e"; }
        }


        private double rangeMax {
            get { return values.Length * values.Length; }
        }

        private const string values = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-.";

    }
}
