﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Google.Charts {
    
    public sealed class SimpleEncoder : DataEncoder {

        internal SimpleEncoder()
            :base()
        {
            this.MissingValue = "_";
            this.SetSeparator = ",";
        }

        public override string EncodeValue(double value, double maxValue, double minValue) {

            if (value > maxValue) {
                throw new ArgumentOutOfRangeException("value", "value must be less than maxValue");
            }

            if (value < minValue)
            {
                throw new ArgumentOutOfRangeException("value", "value must be greater than minValue");
            }

            if (!IsValid(value)) {
                return MissingValue;
            }

            int range = values.Length;
            if (maxValue == minValue)
                minValue = 0;
            double k = range / (maxValue - minValue);
            int index = (int)Math.Round((value-minValue) * k);
            if (index > 0) index--;
            return values[index].ToString();
        }

        public override string Identifier {
            get { return "s"; }
        }

        private const string values = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"; 
    }
}
