﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Globalization;

namespace Google.Charts {

    public sealed class TextEncoder : DataEncoder {

        internal TextEncoder() : base(",") {
            this.MissingValue = "-1";
            this.SetSeparator = "|";
        }

        public override string EncodeValue(double value, double maxValue, double minValue)
        {

            if (value > maxValue) {
                throw new ArgumentOutOfRangeException("value", "value must be less than maxValue");
            }

            if (value < minValue)
            {
                throw new ArgumentOutOfRangeException("value", "value must be greater than minValue");
            }

            if (!IsValid(value)) {
                return MissingValue;
            }

            if (maxValue == minValue)
                minValue = 0;

            double k = range / (maxValue - minValue);
            double relVal = (value - minValue) * k;

            return relVal.ToString("0.0", CultureInfo.InvariantCulture);
        }

        public override string Identifier {
            get { return "t"; }
        }

        const double range = 100.0;
    }
}
