﻿using System;
using System.Collections.Generic;

using System.Drawing;
using System.Diagnostics;

namespace Google.Charts
{
    internal static class Utils
    {
        /// <summary>
        /// Checks for <see langword="null" /> or empty strings, throws exception when string is 
        /// <see langword="null" /> or empty.
        /// </summary>
        /// <param name="value">The value to check.</param>
        /// <param name="parameter">The name of the parameter the value represents.</param>
        /// <exception cref="ArgumentNullException">Thrown when value is <see langword="null" /> or empty.</exception>
        public static void ValidateNotNullOrEmpty(this string value, string parameter)
        {
            ValidateNotNull(value, parameter);

            Debug.Assert(value.Length > 0, String.Format("{0} is empty.", parameter));

            if (value.Length == 0)
            {
                throw new ArgumentException("Parameter is empty.", parameter);
            }
        }

        public static void ValidateNotNull(this object value, string parameter)
        {
            Debug.Assert(value != null, String.Format("{0} is null.", parameter));
            if (value == null) { throw new ArgumentNullException(parameter); }
        }

        /// <summary>
        /// Converts a color to it's (hexadecimal encoded) Html ARGB values.
        /// </summary>
        /// <param name="color">The <see cref="Color" /> to convert.</param>
        /// <returns>An 8-letter string of hexadecimal values in the format AARRGGBB of 
        /// <paramref name="color"/> is not <see langword="null" />; otherwise returns <c>FF000000</c>.</returns>
        public static string ToHtml(this Color? color)
        {
            if (!color.HasValue) { return "000000"; }

            return color.Value.ToHtml();
        }

        public static string ToHtml(this Color color)
        {
            // we ommit the alpha-channel when not explicitly set
            return (color.A != 255 ? color.A.ToString("X2") : "") + color.R.ToString("X2") + color.G.ToString("X2") + color.B.ToString("X2");
        }
    }
}
